from .text_utils import *
from .audio_utils import *
from .seo_utils import *
from .functions import *

# Clear root logger handlers to prevent duplicate console output
logging.getLogger('').handlers.clear()
###json_data,chunk_length_ms=10000,renew=False
def transcribe_all_video_paths(directory=None,
                               output_dir=None,
                               remove_phrases=None,
                               summarizer=None,
                               kw_model=None,
                               uploader=None,
                               domain=None,
                               categories=None,
                               video_url=None,
                               chunk_length_ms=None,
                               chunk_length_diff=None,
                               renew=None):
    logger.info(f"Entering transcribe_all_video_paths")
    directory = directory or os.getcwd()
    paths = glob.glob(path_join(directory, '**', '**'), recursive=True)
    paths = [file_path for file_path in paths if confirm_type(file_path,
                                                              media_types=get_media_types(['video']))]
    video_paths = get_all_file_types(directory=directory, types='video') or get_all_file_types(directory=abs_dirname, types='videos')
    for video_path in video_paths:
        transcribe_video_path(video_path=video_path,
                              output_dir=output_dir,
                              remove_phrases=remove_phrases,
                              summarizer=summarizer,
                              kw_model=kw_model,
                              uploader=uploader,
                              domain=domain,
                              categories=categories,
                              video_url=video_url,
                              chunk_length_ms=chunk_length_ms,
                              chunk_length_diff=chunk_length_diff,
                              renew=renew)
    logger.info(f"Exiting transcribe_all_video_paths")

def transcribe_video_path(video_path,
                          output_dir=None,
                          remove_phrases=None,
                          summarizer=None,
                          kw_model=None,
                          uploader=None,
                          domain=None,
                          categories=None,
                          video_url=None,
                          chunk_length_ms=None,
                          chunk_length_diff=None,
                          renew=None):
    remove_phrases = remove_phrases or []
    output_dir = output_dir if output_dir else make_dirs(directory, 'text_dir')
    logger.info(f"Processing video: {video_path}")
    info = get_info_data(video_path,
                         output_dir=output_dir,
                         remove_phrases=remove_phrases,
                         uploader=uploader,
                         domain=domain,
                         categories=categories,
                         video_url=video_url,
                         chunk_length_ms=chunk_length_ms,
                         chunk_length_diff=chunk_length_diff,
                         renew=renew)
    if not os.path.isfile(info['audio_path']):
        extract_audio_from_video(video_path=info['video_path'],
                                 audio_path=info['audio_path'])
    info = transcribe_audio_file(audio_path=info['audio_path'],
                                 json_data=info,
                                 chunk_length_ms=chunk_length_ms,
                                 chunk_length_diff=chunk_length_diff,
                                 renew=renew)
    info = get_text_and_keywords(info,
                                 summarizer=summarizer,
                                 kw_model=kw_model)
    info = get_seo_data(info,
                        uploader=uploader,
                        domain=domain,
                        categories=categories,
                        video_url=video_url)
    safe_dump_to_file(data=info,
                      file_path=info['info_path'])
    return info

def get_info_data(video_path,
                  output_dir=None,
                  remove_phrases=None,
                  uploader=None,
                  domain=None,
                  categories=None,
                  video_url=None,
                  chunk_length_ms=None,
                  chunk_length_diff=None,
                  renew=None):
    remove_phrases = remove_phrases or []
    dirname = os.path.dirname(video_path)
    basename = os.path.basename(video_path)
    filename, ext = os.path.splitext(basename)
    video_directory = make_dirs(output_dir, filename)
    info_path = os.path.join(video_directory, 'info.json')
    video_text_path = os.path.join(video_directory, 'video_text.json')
    audio_path = os.path.join(video_directory, 'audio.wav')
    video_json_path = os.path.join(video_directory, 'video_json.json')
    categories = categories or {'ai': 'Technology', 'cannabis': 'Health', 'elon musk': 'Business'}
    uploader = uploader or 'The Daily Dialectics'
    domain = domain or 'https://thedailydialectics.com'
    video_url = video_url or url_join(domain,videos)
    chunk_length_ms = if_none_get_def(chunk_length_ms, 50000)
    chunk_length_diff = if_none_get_def(chunk_length_diff, -5)
    renew = if_none_get_def(renew, False)
    info = {}
    if os.path.isfile(info_path):
        info = safe_read_from_json(info_path)
    info['video_path'] = video_path
    info['video_directory'] = video_directory
    info['info_path'] = info_path
    info['filename'] = filename
    info['ext'] = ext
    info['remove_phrases'] = remove_phrases
    info['audio_path'] = audio_path
    info['video_json'] = video_json_path
    info['categories'] = categories
    info['uploader'] = uploader
    info['domain'] = domain
    info['video_url'] = video_url
    info['chunk_length_ms'] = chunk_length_ms
    info['chunk_length_diff'] = chunk_length_diff
    info['renew'] = renew
    safe_dump_to_file(data=info, file_path=info['info_path'])
    return info




import spacy,pytesseract,cv2
from PIL import Image
import numpy as np
import PyPDF2
import argparse
from pdf2image import convert_from_path

from abstract_math import (divide_it,
                           multiply_it)

from abstract_utilities import (timestamp_to_milliseconds,
                                format_timestamp,
                                get_time_now_iso,
                                parse_timestamp,
                                get_logFile,
                                url_join,
                                make_dirs,
                                safe_dump_to_file,
                                safe_read_from_json,
                                read_from_file,
                                write_to_file,
                                os,
                                sys,
                                json,
                                glob,
                                datetime,
                                timedelta,
                                logging,
                                shutil,
                                re,
                                path_join,
                                confirm_type,
                                get_media_types,
                                get_all_file_types,
                                eatInner,
                                eatOuter,
                                eatAll)
                                
                                
logger = get_logFile('vid_to_aud')
logger.debug(f"Logger initialized with {len(logger.handlers)} handlers: {[h.__class__.__name__ for h in logger.handlers]}")

def create_key_value(json_obj, key, value):
    json_obj[key] = json_obj.get(key, value) or value
    return json_obj

def getPercent(i):
    return divide_it(i, 100)

def getPercentage(num, i):
    percent = getPercent(i)
    percentage = multiply_it(num, percent)
    return percentage

def if_none_get_def(value, default):
    if value is None:
        value = default
    return value

def if_not_dir_return_None(directory):
    str_directory = str(directory)
    if os.path.isdir(str_directory):
        return str_directory
    return None



from .functions import logger, create_key_value,cv2, pytesseract, os, shutil, spacy, json, re
from collections import Counter
from .ocr_utils import extract_text_from_image
from moviepy.editor import *
import numpy as np
nlp = spacy.load("en_core_web_sm")

def is_frame_analyzed(frame_file,video_text_data):
    for values in video_text_data:
        frame = values.get("frame")
        if frame_file == frame:
             return True
def determine_remove_text(text,remove_phrases=None):
    remove_phrases=remove_phrases or []
    found = False
    for remove_phrase in remove_phrases:
        if remove_phrase in text:
            found = True
            break
    if found == False:
        return text
def analyze_video_text(video_path,output_dir=None,video_text=None,remove_phrases=None):
    logger.info(f"analyze_video_text: {video_path}")
    remove_phrases=remove_phrases or []
    dirname = os.path.dirname(video_path)
    basename = os.path.basename(video_path)
    filename,ext = os.path.splitext(basename)
    video_text =video_text or []
    output_dir = output_dir or os.path.join(dirname,'./frames')
    os.makedirs(output_dir, exist_ok=True)

    # Load the video
    video = VideoFileClip(video_path)
    duration = video.duration

    # Extract frames every 1 second
    frame_interval = 1  # Adjust as needed
    for t in range(0, int(duration), frame_interval):
        
        frame_path = f"{output_dir}/{filename}_frame_{t}.jpg"
        if not os.path.isfile(frame_path):
            frame = video.get_frame(t)
            cv2.imwrite(frame_path, cv2.cvtColor(np.array(frame), cv2.COLOR_RGB2BGR))

    # Process all extracted frames
    for frame_file in os.listdir(output_dir):
        if frame_file.endswith(".jpg"):
            if is_frame_analyzed(frame_file,video_text) == False:
                
                image_dir = f"{output_dir}/{frame_file}"
                text = extract_text_from_image(image_dir)
                text = determine_remove_text(text,remove_phrases=remove_phrases)

                
                video_text.append({"frame": frame_file, "text": text})
                
    # Clean up (optional)
    return video_text

def extract_keywords_nlp(text, top_n=10):
    doc = nlp(text)
    word_counts = Counter(token.text for token in doc if token.pos_ in ["NOUN", "PROPN"] and not token.is_stop and len(token.text) > 2)
    entity_counts = Counter(ent.text.lower() for ent in doc.ents if len(ent.text.split()) > 1)
    top_keywords = [word for word, _ in (word_counts + entity_counts).most_common(top_n)]
    return top_keywords
def calculate_keyword_density(text, keywords):
    words = text.lower().split()
    return {kw: (words.count(kw.lower()) / len(words)) * 100 for kw in keywords if len(words) > 0}
def get_text_and_keywords(json_data,summarizer=None,kw_model=None):
    json_data['video_text'] = analyze_video_text(json_data.get('video_path'),output_dir=json_data.get('thumbnails_dir'),video_text=json_data.get('video_text'),remove_phrases=json_data.get('remove_phrases'))
    full_text = " ".join([entry['text'] for entry in json_data['audio_text']]).strip()
    json_data['full_text'] = full_text
    json_data['keywords'] = extract_keywords_nlp(full_text, top_n=10)
    if kw_model:
        keybert_keywords = kw_model.extract_keywords(full_text, keyphrase_ngram_range=(1, 3), stop_words='english', top_n=10, use_mmr=True, diversity=0.5)
        combined_keywords = list(set([kw[0] for kw in keybert_keywords] + json_data['keywords']))[:10]
        json_data['combined_keywords'] = combined_keywords
        json_data['keyword_density'] = calculate_keyword_density(full_text, combined_keywords)
    json_data['summary'] = None
    if full_text and summarizer:
        try:
            json_data['summary'] = summarizer(full_text, max_length=160, min_length=40)[0]['summary_text']
        except Exception as e:
            logger.info(f"{e}")
    return json_data



from .functions import *
def preprocess_for_ocr(image_path: str) -> np.ndarray:
    img = cv2.imread(image_path)
    gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)

    # Enhance contrast
    clahe = cv2.createCLAHE(clipLimit=2.0, tileGridSize=(8, 8))
    contrast = clahe.apply(gray)

    # Denoise
    denoised = cv2.bilateralFilter(contrast, d=9, sigmaColor=75, sigmaSpace=75)

    # Thresholding
    thresh = cv2.adaptiveThreshold(
        denoised,
        255,
        cv2.ADAPTIVE_THRESH_GAUSSIAN_C,
        cv2.THRESH_BINARY_INV,
        blockSize=11,
        C=2
    )

    # Sharpen
    sharpen_kernel = np.array([[-1, -1, -1], [-1, 9, -1], [-1, -1, -1]])
    sharpened = cv2.filter2D(thresh, -1, sharpen_kernel)

    return sharpened

def extract_text_from_image(image_path: str) -> str:
    try:
        processed_img = preprocess_for_ocr(image_path)
        pil_img = Image.fromarray(cv2.bitwise_not(processed_img))  # invert for OCR
        text = pytesseract.image_to_string(pil_img, lang='eng')
        return text
    except Exception as e:
        print(f"[OCR Error] {e}")
        return ""



def download_pdf(url: str, output_path: str):
    response = requests.get(url, stream=True)
    if response.status_code == 200:
        with open(output_path, "wb") as file:
            for chunk in response.iter_content(chunk_size=1024):
                file.write(chunk)
        print(f"PDF downloaded successfully: {output_path}")
    else:
        print(f"Failed to download PDF. Status code: {response.status_code}")
is_start = False
# Helper functions (as defined previously)
def preprocess_image(image_path: str, output_path: str) -> None:
    img = cv2.imread(image_path)
    gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
    clahe = cv2.createCLAHE(clipLimit=2.0, tileGridSize=(8, 8))
    contrast = clahe.apply(gray)
    denoised = cv2.bilateralFilter(contrast, d=9, sigmaColor=75, sigmaSpace=75)
    thresh = cv2.adaptiveThreshold(
        denoised, 
        255, 
        cv2.ADAPTIVE_THRESH_GAUSSIAN_C, 
        cv2.THRESH_BINARY_INV, 
        blockSize=11, 
        C=2
    )
    kernel = np.ones((3, 3), np.uint8)
    morph = cv2.dilate(thresh, kernel, iterations=1)
    sharpen_kernel = np.array([[-1, -1, -1], [-1, 9, -1], [-1, -1, -1]])
    sharpened = cv2.filter2D(morph, -1, sharpen_kernel)
    cv2.imwrite(output_path, sharpened)

def convert_image_to_text(image_path: str) -> str:
    try:
        img = Image.open(image_path)
        text = pytesseract.image_to_string(img, lang='eng')
        return text
    except Exception as e:
        print(f"OCR Error: {e}")
        return ""

def clean_text(text: str) -> str:
    text = re.sub(r'\s+', ' ', text)
    text = re.sub(r'[^a-zA-Z0-9\s:.,-]', '', text)
    text = text.strip()
    return text

def write_to_file(file_path: str, contents: str) -> None:
    with open(file_path, 'w', encoding='utf-8') as f:
        f.write(contents)

def process_pdf(main_pdf_path: str, pdf_output_dir: str) -> None:
    """Process a PDF into pages, images, text, preprocessed images, preprocessed text, and cleaned preprocessed text."""
    # Get PDF filename without extension
    pdf_name = os.path.splitext(os.path.basename(main_pdf_path))[0]

    # Create subdirectories for this PDF
    pdf_pages_dir = os.path.join(pdf_output_dir, 'pdf_pages')
    os.makedirs(pdf_pages_dir, exist_ok=True)
    images_dir = os.path.join(pdf_output_dir, 'images')
    os.makedirs(images_dir, exist_ok=True)
    text_dir = os.path.join(pdf_output_dir, 'text')
    os.makedirs(text_dir, exist_ok=True)
    cleaned_text_dir = os.path.join(text_dir, 'cleaned')
    os.makedirs(cleaned_text_dir, exist_ok=True)
    preprocessed_images_dir = os.path.join(pdf_output_dir, 'preprocessed_images')
    os.makedirs(preprocessed_images_dir, exist_ok=True)
    preprocessed_text_dir = os.path.join(pdf_output_dir, 'preprocessed_text')
    os.makedirs(preprocessed_text_dir, exist_ok=True)
    cleaned_preprocessed_text_dir = os.path.join(preprocessed_text_dir, 'cleaned')
    os.makedirs(cleaned_preprocessed_text_dir, exist_ok=True)
    

    pdf_reader = PyPDF2.PdfReader(main_pdf_path)
    num_pages = len(pdf_reader.pages)
    logging.info(f"Processing {pdf_name} with {num_pages} pages")

    for page_num in range(num_pages):
        
            # Split PDF into individual pages
            pdf_writer = PyPDF2.PdfWriter()
            pdf_writer.add_page(pdf_reader.pages[page_num])
            filename = f"{pdf_name}_page_{page_num + 1}"
            basename_pdf= f"{filename}.pdf"
            basename_png= f"{filename}.png"
            preprocessed_basename_png = f"preprocessed_{basename_png}"
            basename_txt= f"{filename}.txt"
            cleaned_basename_txt = f"cleaned_{basename_txt}"
            preprocessed_basename_txt = f"preprocessed_{basename_txt}"
            preprocessed_cleaned_basename_txt = f"preprocessed_cleaned_{basename_txt}"
            page_path = os.path.join(pdf_pages_dir, basename_pdf)
            with open(page_path, 'wb') as f:
                pdf_writer.write(f)

            # Convert PDF page to image
            images = convert_from_path(page_path)
            if images:
                # Save the image
                img_path = os.path.join(images_dir, basename_png)
                images[0].save(img_path, 'PNG')

                # Extract text directly from the image
                text = convert_image_to_text(img_path)
                txt_path = os.path.join(text_dir, basename_txt)
                write_to_file(file_path=txt_path, contents=text)

                # Clean the extracted text
                cleaned_text = clean_text(text)
                cleaned_text_path = os.path.join(cleaned_text_dir, cleaned_basename_txt)
                write_to_file(file_path=cleaned_text_path, contents=cleaned_text)

                # Preprocess the image
                
                preprocessed_img_path = os.path.join(preprocessed_images_dir, preprocessed_basename_png)
                preprocess_image(img_path, preprocessed_img_path)

                # Extract text from the preprocessed image
                preprocessed_text = convert_image_to_text(preprocessed_img_path)
                
                preprocessed_txt_path = os.path.join(preprocessed_text_dir, preprocessed_basename_txt)
                write_to_file(file_path=preprocessed_txt_path, contents=preprocessed_text)

                # Clean the preprocessed text
                preprocessed_cleaned_text = clean_text(preprocessed_text)
                
                preprocessed_cleaned_txt_path = os.path.join(cleaned_preprocessed_text_dir, preprocessed_cleaned_basename_txt)
                write_to_file(file_path=preprocessed_cleaned_txt_path, contents=preprocessed_cleaned_text)
                    
                logging.info(f"Processed page {page_num + 1} of {pdf_name}")



from .functions import (logger,
                        create_key_value,
                        timestamp_to_milliseconds,
                        format_timestamp,
                        parse_timestamp,
                        get_time_now_iso,
                        shutil,
                        os,
                        safe_dump_to_file,
                        if_not_dir_return_None,
                        if_none_get_def,
                        getPercentage,
                        getPercent,
                        create_key_value)
from moviepy.editor import *
import moviepy.editor as mp
import speech_recognition as sr
from pydub import AudioSegment
r = sr.Recognizer()
def get_elapsed_time(start_time, end_time):
    elapsed = end_time - start_time
    if end_time < start_time:
        elapsed = 0
    return elapsed

def get_time_diff(start_time, end_time, chunk_length_diff):
    elapsed = get_elapsed_time(start_time, end_time)
    percentage = getPercentage(elapsed, chunk_length_diff)
    new_start = subtract_it(start_time, percentage)
    if new_start <= 0:
        new_start = start_time
    return new_start

def check_if_transcribed_block(audio_text, start_time, end_time):
    for entry in audio_text:
        entry_start = timestamp_to_milliseconds(entry.get("start_time", "0:00"))
        entry_end = timestamp_to_milliseconds(entry.get("end_time", "0:00"))
        if entry_start <= start_time and entry_end >= end_time and entry.get("text", "").strip():
            return True
    uncovered_start, uncovered_end = start_time, end_time
    for entry in audio_text:
        entry_start = timestamp_to_milliseconds(entry.get("start_time", "0:00"))
        entry_end = timestamp_to_milliseconds(entry.get("end_time", "0:00"))
        if not entry.get("text", "").strip():
            continue
        if entry_start <= uncovered_start < entry_end:
            uncovered_start = entry_end
        if entry_start < uncovered_end <= entry_end:
            uncovered_end = entry_start
    return (uncovered_start, uncovered_end)

def get_chunks(audio, chunk_length_ms=None, chunk_length_diff=None):
    chunk_length_ms = if_none_get_def(chunk_length_ms, 50000)
    chunk_length_diff = if_none_get_def(chunk_length_diff, -5)
    chunks = [audio[i:i + (chunk_length_ms - chunk_length_diff)] for i in range(0, len(audio), chunk_length_ms)]
    return chunks

def get_text(audio_data):
    try:
        text = r.recognize_google(audio_data)
    except:
        text = ""
    return text

def get_audio_chunk(start_time, end_time, audio, chunk_path=None):
    chunk_path = chunk_path or f"{os.getcwd()}/chunk_partial.wav"
    audio[start_time:end_time].export(chunk_path, format="wav")
    with sr.AudioFile(chunk_path) as source:
        r.adjust_for_ambient_noise(source)
        audio_data = r.record(source)
        return audio_data

def get_chunk_period(i, audio_text, chunk_length_ms, chunk_length_diff, audio_len, renew=False):
    start_time = i * chunk_length_ms
    end_time = min((i + 1) * chunk_length_ms, audio_len)
    if i == 0:
        end_time += chunk_length_diff
    result = (start_time, end_time)
    if not renew:
        result = check_if_transcribed_block(audio_text, start_time, end_time)
    return result

def get_audio_text(start_time, end_time, audio, chunk_path=None):
    audio_data = get_audio_chunk(start_time=start_time,
                                 end_time=end_time,
                                 audio=audio,
                                 chunk_path=chunk_path)
    txt = get_text(audio_data)
    return txt

def get_audio_text_data(i,
                        audio,
                        json_data=None,
                        chunk_length_ms=None,
                        chunk_length_diff=None,
                        file_path=None,
                        renew=False,
                        output_directory=None):
    json_data = json_data or {}
    json_data = create_key_value(json_data, 'audio_text', [])
    output_directory = if_none_get_def(if_not_dir_return_None(output_directory), os.getcwd())
    chunk_path = os.path.join(output_directory, f"chunk_{i}_partial.wav")
    result = get_chunk_period(i,
                              audio_text=json_data['audio_text'],
                              chunk_length_ms=chunk_length_ms,
                              chunk_length_diff=chunk_length_diff,
                              audio_len=len(audio),
                              renew=renew)
    if isinstance(result, tuple):
        logging.info(f"transcribing audio chunk {i}, start_time:{result[0]}, end_time:{result[1]}")
        text = get_audio_text(start_time=result[0],
                              end_time=result[1],
                              audio=audio,
                              chunk_path=chunk_path)
        json_data['audio_text'].append({
            "start_time": format_timestamp(result[0]),
            "end_time": format_timestamp(result[1]),
            "text": text
        })
        if file_path:
            safe_dump_to_file(json_data, file_path)
    else:
        logging.info(f"audio text already transcribed for chunk {i}")
    if os.path.isfile(chunk_path):
        try:
            os.remove(chunk_path)
        except OSError:
            logger.warning(f"Failed to remove chunk file: {chunk_path}")
    return json_data

def get_audio_text_jsons(audio_path,
                         json_data=None,
                         chunk_length_ms=None,
                         chunk_length_diff=None,
                         output_directory=None,
                         file_path=None,
                         renew=False):
    json_data = json_data or {}
    audio = AudioSegment.from_wav(audio_path)
    chunks = get_chunks(audio,
                        chunk_length_ms=chunk_length_ms,
                        chunk_length_diff=chunk_length_diff)
    len_chunks = len(chunks)
    for i in range(len_chunks):
        json_data = get_audio_text_data(i,
                                        audio=audio,
                                        json_data=json_data,
                                        chunk_length_ms=chunk_length_ms,
                                        chunk_length_diff=chunk_length_diff,
                                        file_path=file_path,
                                        renew=renew,
                                        output_directory=output_directory)
        logging.info(f"chunk {i} of {len_chunks} processed")
    return json_data

def transcribe_audio_file(audio_path=None, json_data=None, video_path=None, chunk_length_ms=8000, chunk_length_diff=-5, renew=False):
    logger.info(f"transcribe_audio_file: {audio_path}")
    json_data = json_data or {}
    audio_path = audio_path or json_data.get('audio_path')
    video_path = video_path or json_data.get('video_path')
    json_data = create_key_value(json_data, 'video_path', video_path)
    file_path = json_data.get('info_path') or os.path.join(os.getcwd(), 'info.json')
    json_data = create_key_value(json_data, 'info_path', file_path)
    info_directory = json_data.get('info_directory') or os.path.dirname(file_path)
    json_data = create_key_value(json_data, 'info_directory', info_directory)
    json_data = create_key_value(json_data, 'audio_text', [])
    if not audio_path and not video_path:
        logger.info(f"no audio_path and no video_path: {file_path}")
        return json_data
    if not audio_path and video_path:
        audio_path = os.path.join(os.path.dirname(video_path), 'audio.wav')
        logging.info(f"audio path derived from video_path:{video_path}")
        json_data = create_key_value(json_data, 'audio_path', audio_path)
    if not os.path.isfile(audio_path) and video_path:
        extract_audio_from_video(video_path, audio_path)
        logging.info(f"extracted audio from video:{audio_path}")
    temp_file_path = file_path.replace('.json', '_temp.json')
    json_data = get_audio_text_jsons(audio_path=audio_path,
                                     json_data=json_data,
                                     chunk_length_ms=chunk_length_ms,
                                     chunk_length_diff=chunk_length_diff,
                                     output_directory=info_directory,
                                     file_path=temp_file_path,
                                     renew=renew)
    if os.path.isfile(temp_file_path):
        try:
            shutil.move(temp_file_path, file_path)
            logging.info(f"temp_info_path transfered to info_path:{file_path}")
        except OSError as e:
            logger.error(f"Failed to move temp file to {file_path}: {e}")
    return json_data
def extract_audio_from_video(video_path, audio_path):
    """Extract audio from a video file using moviepy."""
    try:
        logger.info(f"Extracting audio from {video_path} to {audio_path}")
        video = mp.VideoFileClip(video_path)
        video.audio.write_audiofile(audio_path)
        video.close()
        if not os.path.exists(audio_path):
            raise FileNotFoundError(f"Audio file {audio_path} was not created.")
        logging.info(f"Audio extracted successfully: {audio_path}")
        return audio_path
    except Exception as e:
        logging.error(f"Error extracting audio from {video_path}: {e}")
        return None
    
def get_voice(voice, text=None):
    """Save transcribed text to a file."""
    text = text or ''
    if voice:
        text = text + '\n' + str(voice) if text else str(voice)
    return text
def stitch_text(last_text, text):
    og_text = text
    text = eatInner(text, [' ', '\n', '\t'])
    last_text = eatAll(last_text, [' ', '\n', '\t'])
    last_text_spl = last_text.split(' ')[-1]
    end_match = False
    spl_len = len(last_text_spl)
    text_len = len(text)
    j = 0
    for i in range(spl_len):
        k = i - j
        is_valid = k < text_len
        if end_match == True or not is_valid:
            if last_text_spl[j:i] != text[:k] or spl_len == i + 1 or not is_valid:
                if not is_valid and k > 0:
                    k = k - 1
                og_text = text[k:]
                break
        if end_match == False and last_text_spl[i] == text[0]:
            j = i
            end_match = True
    return og_text
